<?php

namespace WCYandexTaxiDeliveryPlugin;

defined( 'ABSPATH' ) || exit;

use YandexTaxi\Delivery\Entities\ClaimItem\ClaimItem;
use YandexTaxi\Delivery\Entities\RoutePoint\Address;
use YandexTaxi\Delivery\Exceptions\ValidationError;
use YandexTaxi\Delivery\GeoCoding\Point;
use YandexTaxi\Delivery\GeoCoding\YandexMaps\YandexGeoCoder;
use YandexTaxi\Delivery\GeoCoding\GoogleMaps\GoogleGeoCoder;
use YandexTaxi\Delivery\GeoCoding\Dadata\DadataGeoCoder;
use YandexTaxi\Delivery\YandexApi\Exceptions\YandexApiException;
use YandexTaxi\Delivery\YandexApi\Resources\PriceChecker;

/**
 * Class ClientPackageService
 *
 * @package WCYandexTaxiDeliveryPlugin
 */
class ClientPackageService extends BasePackageService {
	/** @var PriceChecker */
	private $priceChecker;

	/** @var YandexGeoCoder|GoogleGeoCoder|DadataGeoCoder */
	private $geoCoder;

	public function __construct() {
		$this->priceChecker = Container::get( PriceChecker::class );
		$this->geoCoder     = Container::get( ygo_get_geocoder_classname() );
	}

	/**
	 * @param array $package
	 *
	 * @return float|null
	 * @throws ValidationError
	 * @throws YandexApiException
	 */
	public function calculateSum( array $package ): ?float {
		$items  = $this->prepareItems( $package['contents'] );
		$source = $this->getSource();

		$destination = $this->prepareDestination( $package );

		$checkPriceResult = $this->priceChecker->calculate( $items, [ $source, $destination ] );

		return $checkPriceResult->getPrice()->getValue();
	}

	/**
	 * @param array $products
	 *
	 * @return ClaimItem[]
	 */
	private function prepareItems( array $products ): array {
		$items = [];

		foreach ( $products as $item ) {
			$items[] = $this->prepareItem( $item['data'], null, $item['quantity'] );
		}

		return $items;
	}

	private function getSource(): Address {
		$warehouse = DefaultWarehouseFinder::find();

		if ( empty( $warehouse ) ) {
			throw new ValidationError( 'Default warehouse is not found' );
		}

		return new Address( $warehouse->getAddress(), $warehouse->getLat(), $warehouse->getLon() );
	}

	private function prepareDestination( array $package ): Address {
		$addressLine = $this->getAddressLineFromPackage( $package );
		$addressLine = trim( $addressLine );

		if ( empty( $addressLine ) ) {
			throw new ValidationError( 'Address is not filled' );
		}

		if ( isset( WC()->session ) ) {
			$stored = WC()->session->get( 'last_address' );

			if ( ! is_null( $stored ) && isset( $stored['address'] ) && $stored['address'] === $addressLine ) {
				return new Address( $addressLine, $stored['lat'], $stored['lon'] );
			}
		}

		$point = $this->geoCoder->decode( $addressLine );

		$this->saveAddressDataInSession( $addressLine, $point );

		return new Address( $addressLine, $point->getLat(), $point->getLon() );
	}

	private function saveAddressDataInSession( string $address, Point $point ): void {
		if ( ! isset( WC()->session ) ) {
			return;
		}

		WC()->session->set( 'last_address', [
			'address' => $address,
			'lat'     => $point->getLat(),
			'lon'     => $point->getLon(),
		] );
	}

	private function getAddressLineFromPackage( array $package ): string {
		$parts = $package['destination'];
		if ( empty( $parts['city'] ) ) {
			return "-";
		}
		$country = WC()->countries->countries[ sanitize_text_field( $parts['country'] ) ];

		return implode( ', ', array_filter( [
			$country,
			sanitize_text_field( $parts['city'] ),
			sanitize_text_field( $parts['address'] ),
		] ) );
	}
}
